/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */

const nsIFilePicker = Components.interfaces.nsIFilePicker;

with (XPCOMUtils) {
  var ImportExport = {
    /**
     * Display a dialog to select the file to import.
     * @returns {string} The path of the selected file.
     */
    importPath: function() {
      var fp = CCIN("@mozilla.org/filepicker;1", "nsIFilePicker");
      fp.init(window, "File to import", nsIFilePicker.modeOpen);
      fp.appendFilters(nsIFilePicker.filterAll);
      
      var rv = fp.show();
      if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
        return fp.file.path;
      }
      
      return undefined;
    },
    
    /**
     * This function parse a financial file.
     * @argument path {string} The path of the file to parse
     * @returns {object} The parsing result
     */
    parse: function() {
      var pEscudo = window.arguments[0].top;
      
      var f = CCIN("@mozilla.org/file/local;1", "nsILocalFile");
      f.initWithPath(document.getElementById("path").value);
      
      if (f.exists() == true) {
        fis = CCIN("@mozilla.org/network/file-input-stream;1", "nsIFileInputStream");
        fis.init(f, 0x01, 0444, 0);

        var is = CCIN("@mozilla.org/intl/converter-input-stream;1", "nsIConverterInputStream");
        is.init(fis, "UTF-8", 1024, 0xFFFD);
        is.QueryInterface(Components.interfaces.nsIUnicharLineInputStream);

        if (is instanceof Components.interfaces.nsIUnicharLineInputStream) {
          var line = {};
          var cont = is.readLine(line);

          //retrieve the index for the differente values of escudo
          var header = line.value.split(/;/);
          header = header.map(function(str) {return str.replace(/"/g, "")});
          var offsetDate = header.indexOf("Date");
          var offsetPayee = header.indexOf("Tiers");
          var offsetTags = header.indexOf("Sous-catégories");
          var offsetCredit = header.indexOf("Crédit");
          var offsetDebit = header.indexOf("Débit");

          do {
            cont = is.readLine(line);
            
            var data = line.value.split(/;/);
            data = data.map(function(str) {return str.replace(/"/g, "")});
            
            //Create the fund
            pEscudo.Accounts.getOrCreateAccount(f.leafName);

            var month, day;
            var re = /(\d{1,2})\/(\d{1,2})\/(\d{2,4})/.exec(data[offsetDate]);
            if (re != null) {
              month = re[2].length == 1 ? "0" + re[2] : re[2];
              day = re[1].length == 1 ? "0" + re[1] : re[1];
            }

            pEscudo.Operations.insert(f.leafName,
                                      re == null ? null : re[3] + "-" + month + "-" + day,
                                      data[offsetCredit].replace(',', '.'),
                                      data[offsetDebit].replace(',', '.'),
                                      data[offsetTags],
                                      data[offsetPayee]);
          } while (cont);
        }
        is.close()
        fis.close();
      }
    },
    
    /**
     * Stop the import process
     */
    abort :function() {
      return true;
    }
  }
}
