/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */


//Unique global variable for escudo
var EscudoLib = XPCOMUtils;

(function() {
  const STRING_BUNDLE = "strings_escudo";
  const PrefService =  this.CCSV("@mozilla.org/preferences-service;1", "nsIPrefBranch");
  const StorageService = this.CCSV("@mozilla.org/storage/service;1", "mozIStorageService");


  /*--------------------------------------------------
  Localization
  --------------------------------------------------*/
  function $STR(name) {
    return document.getElementById(STRING_BUNDLE).getString(name);
  }

  function $STRF(name, args) {
    return document.getElementById(STRING_BUNDLE).getFormattedString(name, args);
  }

  this.$STRF = $STRF;
  this.$STR = $STR;


  /*--------------------------------------------------
  HELPER
  --------------------------------------------------*/
  /**
   * Returns the value of the tree cell
   * @argument tree {string} id of the tree
   * @argument column {string} if of the column
   * @returns {string} the cell value.
   */
  this.getTreeCellValue = function(tree, column) {
    var Tree = document.getElementById(tree);
    var Column = Tree.columns.getNamedColumn(column);
    return Tree.view.getCellText(Tree.currentIndex, Column);
  };

  /**
   * Returns the value of the tree cell
   * @argument tree {string} id of the tree
   * @argument column {string} if of the column
   * @returns {string} the cell value.
   */
  this.getTreeCellValueById = function(tree, column, id) {
    var Tree = document.getElementById(tree);
    var Column = Tree.columns.getNamedColumn(column);
    return Tree.view.getCellText(id, Column);
  };
  
  /*--------------------------------------------------
  Database
  --------------------------------------------------*/
  var connection = undefined;
  var statements = [];

  /**
   * Returns a compiled statement
   * @argument id {string} The statement unique identifier
   * @returns {} The compiled statement or undefined
   */
  this.getStatement = function(query) {
    if (connection === undefined) {
      this.connect();
    }
    let ret = connection.createStatement(query);
    statements.push(ret);
    return ret;
  };

  /**
   * Connect to the sqlite data base in the profile directory.
   * @returns {Boolean} Returns true if the connection succeed, false otherwise.
   */
  this.connect = function() {
    var db = this.getDB();
    dump("DB: " + db + "\n");

    if (db != null && db.exists() == true) {
      connection = StorageService.openDatabase(db);
      
      return true;
    }
    throw $STR("err_dbIncorrect");
    return false;
  };
  
  /**
   * This function is used to close correctly the sqlite database. It finalizes
   * each database statement before closinf the connection.
   */
  this.close = function() {
    dump("Close each statement (" + statements.length + ").\n");
    for (var i = 0; i < statements.length; i++) {
      statements[i].finalize();
    }
    statements = [];

    dump("Close the database connection.\n");
    if (connection != undefined) {
      dump("Closing the database connection.\n");
      connection.close();
    }
    dump("Database connection closed.\n");
  };

  this.lastInsertRowID = function() {
    return connection.lastInsertRowID;
  };

  /**
   * This function returns the Database for the application. The Database is in
   * the profile directory. This function does not check the existance of the
   * database
   * @returns {nsIFile} The file corresponding to the current DB
   */
  this.getDB = function() {
    var value = PrefService.getComplexValue("escudo.db", Components.interfaces.nsIRelativeFilePref);
    return value.file;
  };

  /*--------------------------------------------------
  Observer
  --------------------------------------------------*/
  const ObserverService = this.CCSV("@mozilla.org/observer-service;1", "nsIObserverService");

  this.addObserver = function(topic, observer) {
    ObserverService.addObserver(observer, topic, false);
  };

  this.removeObserver = function(topic, observer) {
    ObserverService.removeObserver(observer, topic);
  };

  this.notifyObserver = function(topic, data) {
    ObserverService.notifyObservers(undefined, topic, data);
  }
}).apply(EscudoLib);
