/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */

with (EscudoLib) {
  top.Escudo.Operations =  {
    d_op: "DELETE FROM operation WHERE id=:op_id",
    i_op: "INSERT INTO operation (\"expense\", \"income\", \"date\", \"payee\", \"account\") VALUES (:expense, :income, :date, :payee, :account)",
    u_op: "UPDATE operation SET date=:date, income=:income, expense=:expense WHERE id=:op_id",
    s_allOp: "SELECT timestamp, ROUND(total(income - expense), 2) AS total FROM view_operations WHERE timestamp is not null GROUP BY date ORDER BY date",

    u_income: "UPDATE operation SET income=:income WHERE id=:op_id",
    u_expense: "UPDATE operation SET expense=:expense WHERE id=:op_id",
    u_date: "UPDATE operation SET date=:date WHERE id=:op_id",
    u_payee: "UPDATE operation SET payee=:payee WHERE id=:op_id",
    
    /****************************************
     * Initialize / Shutdown
     */
    init: function() {
      addObserver("refresh", this.Observer);
      addObserver("invalid", this.Observer);
      
      window.addEventListener("load", function() {
        let tree = document.getElementById("tree-op");
        tree.addEventListener("DOMAttrModified", function(event) {
          try {
            if (tree.editingRow != -1 && event.attrName == "label") {
              top.Escudo.Operations.update(tree.editingColumn.id,
                                           tree.editingRow,
                                           event.newValue);
            }
          } catch(e) { dump(e+"\n"); }
        }, true);
      }, true);
    },

    /****************************************
     * Observer
     */
    Observer : {
      observe: function(subject, topic, data) {
        let tree = document.getElementById("tree-op");
        let param = document.getElementById("param-opAccount");

        if (topic == "invalid") { param.textContent = data; }
        tree.builder.rebuild();
      }
    },

    /****************************************
     * Database access
     */
    remove: function(id) {
      if (confirm($STR("conf_delLedger")) == true) {
        var value = getTreeCellValue("tree-op", "op-id");

        let statement = getStatement(this.d_op);
        statement.params.op_id = value;
        
        try {
          statement.execute();
        } finally {
          statement.reset();
        }
        notifyObserver("refresh", null);
      }
    },

    update: function(column, row, value) {
      dump("newValue=" + value + " editingRow=" + row + " - " + "editingColumn=" + column + "\n");

      var statement = null;
      var opId = getTreeCellValueById("tree-op", "op-id", row);
      switch (column) {
        case "op-date":
          statement = getStatement(this.u_date);
          statement.params.date = value;
          break;
        case "op-income":
          statement = getStatement(this.u_income);
          statement.params.income = value;
          break;
        case "op-expense":
          statement = getStatement(this.u_expense);
          statement.params.expense = value;
          break;
        case "op-tag":
          top.Escudo.Tags.deleteAllConnections(opId);
          if (value != "") {
            value.split(",").forEach(function(element, index, array) {
              top.Escudo.Tags.connect(opId, element);
            });
          }
          //Be careful !!!!
          return;
        case "op-payee":
          statement = getStatement(this.u_payee);
          statement.params.payee = top.Escudo.Payees.getOrCreatePayee(value);
          break;
      }

      statement.params.op_id = opId;
      
      statement.executeAsync({
        handleResult: function(aResultSet) { },
        handleError: function(aError) { },
        handleCompletion: function(aReason) {
          if (aReason == Components.interfaces.mozIStorageStatementCallback.REASON_FINISHED) {
            notifyObserver("refresh", null);
          }
        }
      });
    },

    insertFromIHM: function() {
      let date = document.getElementById("datepicker-addDate").value;
      let debit = document.getElementById("textbox-addExpense").value;
      let credit = document.getElementById("textbox-addIncome").value;
      let account = document.getElementById("texbox-menu").value;
      let tags = document.getElementById("op-tags").value;
      let payee = document.getElementById("texbox-addOp-Payee").value;

      this.insert(account, date, credit, debit, tags, payee)
    },

    insert: function(account, date, credit, debit, tags, payee) {
      //get the different values
      account = top.Escudo.Accounts.getOrCreateAccount(account);
      payee = top.Escudo.Payees.getOrCreatePayee(payee);

      let statement = getStatement(this.i_op);
      statement.params.expense = debit;
      statement.params.income = credit;
      statement.params.date = date;
      statement.params.payee = payee;
      statement.params.account = account;

      try {
        statement.execute();
        
        var opId = lastInsertRowID();
        tags.split(", ").forEach(function(element, index, array) {
          top.Escudo.Tags.connect(opId, element);
        });
  
      } finally {
        statement.reset();
      }
      notifyObserver("refresh", null);
    },

    /**
     */
    getAllOp: function() {
      let statement = getStatement(this.s_allOp);

      var balance = [];
      try {
        var prevAmount = 0;
        while (statement.executeStep()) {
          balance.push([statement.getInt64(0), new Number((prevAmount + statement.getDouble(1)).toFixed(2))]);
          prevAmount += statement.getDouble(1);
        }
      } finally {
        statement.reset();
      }
      return balance;
    },
    
    storeTag: function() {
      var sep = "";
      var opTag = document.getElementById("op-tags");
      var tag = document.getElementById("texbox-addOp-Tag");
      
      if (opTag.value != "") {
        sep = ", ";
      }
      opTag.value += sep + tag.value.trim();
      tag.value = "";
    },
    
    clearTag: function() {
      document.getElementById("op-tags").value = "";
    }
    
  }
  //init
  top.Escudo.Operations.init();
}
