/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */


with (EscudoLib) {
  top.Escudo.Tags =  {
    i_tag: "INSERT INTO tag(\"tag\") VALUES (:tag)",
    s_tag: "SELECT id FROM tag WHERE tag=:tag",
    s_pattern: "SELECT tag FROM tag WHERE tag LIKE :tag ORDER BY tag",
    c_tag: "INSERT INTO j_opTag(\"operation\", \"tag\") VALUES(:op, :tag)",
    d_tag: "DELETE FROM j_opTAG WHERE operation=:op_id",
    
    /****************************************
     * Initialize / Shutdown
     */
    init: function() {
      addObserver("invalid", this.Observer);
      addObserver("refresh", this.Observer);
    },

    Observer : {
      observe: function(subject, topic, data) {
        var tree = document.getElementById("tree-tags");
        var param = document.getElementById("param-tagsAccount");

        if (topic == "invalid") { param.textContent = data; }
        tree.builder.rebuild();
      }
    },

    /****************************************
     * Helper function
     */
    getTagMatchingPattern: function(pattern) {
      let statement = getStatement(this.s_pattern);
      statement.params.tag = "%" + pattern + "%";

      var values = [];
      try {
        while (statement.executeStep()) {
          values.push(statement.getString(0));
        }
      } finally {
        statement.reset();
      }
      return values;
    },

    getTagId: function(tag) {
      let statement = getStatement(this.s_tag);
      statement.params.tag = tag;

      var id = undefined;
      try {
        while (statement.executeStep()) {
          id = statement.getInt32(0);
        }
      } finally {
        statement.reset();
      }
      return id;
    },

    getOrCreateTag: function(tag) {
      var id = this.getTagId(tag);
      if (id == undefined) { id = this.insert(tag); }
      return id;
    },

    insert: function(tag) {
      let statement = getStatement(this.i_tag);
      statement.params.tag = tag;

      try {
        statement.execute();
      } finally {
        statement.reset();
      }
      return lastInsertRowID();
    },
    
    deleteAllConnections: function(opId) {
      let statement = getStatement(this.d_tag);
      statement.params.op_id = opId;

      statement.execute();
      statement.reset();
    },
    
    /**
     * This function connects a tag to an operation. It uses the
     * operation id and creates the tag if necessary.
     * @argument tag {String} The tag to create
     * @argument operation {integer} the operation id
     */
    connect: function(operation, tag) {
      var tag_id = this.getOrCreateTag(tag);

      dump("Connect " + operation + " to " + tag + "\n");
      let statement = getStatement(this.c_tag);
      statement.params.op = operation;
      statement.params.tag = tag_id;

      try {
        statement.execute();
      } finally {
        statement.reset();
      }
      return lastInsertRowID();
    }
  }
  top.Escudo.Tags.init();
}
